@@ -1,3 +1,14 @@
+#include <ctype.h>
+#include <dirent.h>
+#include <glib.h>
+#include <regex.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include "file.h"
+#include "utils.h"
 #include "directorytree.h"
 
 /*
@@ -14,7 +25,7 @@ typedef void (*TimeoutCallback)(void);
 
 FileSystemEntry *createEntry(const char *name, int isDirectory, FileSystemEntry *parent)
 {
-        FileSystemEntry *newEntry = (FileSystemEntry *)malloc(sizeof(FileSystemEntry));
+        FileSystemEntry *newEntry = malloc(sizeof(FileSystemEntry));
         if (newEntry != NULL)
         {
                 newEntry->name = strdup(name);
@@ -56,7 +67,7 @@ void setFullPath(FileSystemEntry *entry, const char *parentPath, const char *ent
 
         size_t fullPathLength = strnlen(parentPath, MAXPATHLEN) + strnlen(entryName, MAXPATHLEN) + 2; // +2 for '/' and '\0'
 
-        entry->fullPath = (char *)malloc(fullPathLength);
+        entry->fullPath = malloc(fullPathLength);
         if (entry->fullPath == NULL)
         {
                 return;
@@ -85,6 +96,109 @@ void freeTree(FileSystemEntry *root)
         free(root);
 }
 
+int naturalCompare(const char *a, const char *b)
+{
+        while (*a && *b)
+        {
+                if (isdigit(*a) && isdigit(*b))
+                {
+                        // Compare numerically
+                        char *endA, *endB;
+                        long numA = strtol(a, &endA, 10);
+                        long numB = strtol(b, &endB, 10);
+
+                        if (numA != numB)
+                        {
+                                return numA - numB;
+                        }
+
+                        // Move pointers past the numeric part
+                        a = endA;
+                        b = endB;
+                }
+                else
+                {
+                        if (*a != *b)
+                        {
+                                return *a - *b;
+                        }
+
+                        a++;
+                        b++;
+                }
+        }
+
+        // If all parts so far are equal, shorter string should come first
+        return *a - *b;
+}
+
+int compareLibEntries(const struct dirent **a, const struct dirent **b)
+{
+        // All strings need to be uppercased or already uppercased characters will come before all lower-case ones
+        char *nameA = stringToUpper((*a)->d_name);
+        char *nameB = stringToUpper((*b)->d_name);
+
+        if (nameA[0] == '_' && nameB[0] != '_')
+        {
+                free(nameA);
+                free(nameB);
+                return 1;
+        }
+        else if (nameA[0] != '_' && nameB[0] == '_')
+        {
+                free(nameA);
+                free(nameB);
+                return -1;
+        }
+
+        int result = naturalCompare(nameA, nameB);
+
+        free(nameA);
+        free(nameB);
+
+        return result;
+}
+
+int compareLibEntriesReversed(const struct dirent **a, const struct dirent **b)
+{
+        int result = compareLibEntries(a, b);
+
+        return -result;
+}
+
+int compareEntryNatural(const void *a, const void *b)
+{
+        const FileSystemEntry *entryA = *(const FileSystemEntry **)a;
+        const FileSystemEntry *entryB = *(const FileSystemEntry **)b;
+
+        char *nameA = stringToUpper(entryA->name);
+        char *nameB = stringToUpper(entryB->name);
+
+        if (nameA[0] == '_' && nameB[0] != '_')
+        {
+                free(nameA);
+                free(nameB);
+                return 1;
+        }
+        else if (nameA[0] != '_' && nameB[0] == '_')
+        {
+                free(nameA);
+                free(nameB);
+                return -1;
+        }
+
+        int result = naturalCompare(nameA, nameB);
+
+        free(nameA);
+        free(nameB);
+        return result;
+}
+
+int compareEntryNaturalReversed(const void *a, const void *b)
+{
+        return -compareEntryNatural(a, b);
+}
+
 int removeEmptyDirectories(FileSystemEntry *node)
 {
         if (node == NULL)
@@ -159,6 +273,11 @@ int readDirectory(const char *path, FileSystemEntry *parent)
         {
                 struct dirent *entry = entries[i];
 
+                if (entry == NULL)
+                {
+                        continue;
+                }
+
                 if (entry->d_name[0] != '.' && strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0)
                 {
                         char childPath[MAXPATHLEN];
@@ -170,11 +289,11 @@ int readDirectory(const char *path, FileSystemEntry *parent)
                                 continue;
                         }
 
-                        int isDirectory = true;
+                        int isDir = true;
 
                         if (S_ISREG(fileStats.st_mode))
                         {
-                                isDirectory = false;
+                                isDir = false;
                         }
 
                         char exto[100];
@@ -182,18 +301,16 @@ int readDirectory(const char *path, FileSystemEntry *parent)
 
                         int isAudio = match_regex(&regex, exto);
 
-                        if (isAudio == 0 || isDirectory)
+                        if (isAudio == 0 || isDir)
                         {
-                                FileSystemEntry *child = createEntry(entry->d_name, isDirectory, parent);
 
-                                if (entry != NULL)
-                                {
-                                        setFullPath(child, path, entry->d_name);
-                                }
+                                FileSystemEntry *child = createEntry(entry->d_name, isDir, parent);
+
+                                setFullPath(child, path, entry->d_name);
 
                                 addChild(parent, child);
 
-                                if (isDirectory)
+                                if (isDir)
                                 {
                                         numEntries++;
                                         numEntries += readDirectory(childPath, child);
@@ -296,17 +413,17 @@ FileSystemEntry *reconstructTreeFromFile(const char *filename, const char *start
 
         while (fgets(line, sizeof(line), file))
         {
-                int id, parentId, isDirectory;
+                int id, parentId, isDir;
                 char name[256];
 
-                if (sscanf(line, "%d\t%255[^\t]\t%d\t%d", &id, name, &isDirectory, &parentId) == 4)
+                if (sscanf(line, "%d\t%255[^\t]\t%d\t%d", &id, name, &isDir, &parentId) == 4)
                 {
                         if (id >= nodesCapacity)
                         {
                                 oldCapacity = nodesCapacity;
                                 nodesCapacity = id + 100;
-                                FileSystemEntry **tempNodes = resizeNodesArray(nodes, oldCapacity, nodesCapacity);
-                                if (!tempNodes)
+                                FileSystemEntry **tmpNodes = resizeNodesArray(nodes, oldCapacity, nodesCapacity);
+                                if (!tmpNodes)
                                 {
                                         perror("Failed to resize nodes array");
 
@@ -321,9 +438,9 @@ FileSystemEntry *reconstructTreeFromFile(const char *filename, const char *start
                                         }
                                         free(nodes);
                                         fclose(file);
-                                        exit(EXIT_FAILURE);
+                                        exit(1);
                                 }
-                                nodes = tempNodes;
+                                nodes = tmpNodes;
                         }
 
                         FileSystemEntry *node = malloc(sizeof(FileSystemEntry));
@@ -332,11 +449,11 @@ FileSystemEntry *reconstructTreeFromFile(const char *filename, const char *start
                                 perror("Failed to allocate node");
 
                                 fclose(file);
-                                exit(EXIT_FAILURE);
+                                exit(1);
                         }
                         node->id = id;
                         node->name = strdup(name);
-                        node->isDirectory = isDirectory;
+                        node->isDirectory = isDir;
                         node->isEnqueued = 0;
                         node->children = node->next = node->parent = NULL;
                         nodes[id] = node;
@@ -361,7 +478,7 @@ FileSystemEntry *reconstructTreeFromFile(const char *filename, const char *start
 
                                 setFullPath(node, nodes[parentId]->fullPath, node->name);
 
-                                if (isDirectory)
+                                if (isDir)
                                         *numDirectoryEntries = *numDirectoryEntries + 1;
                         }
                         else
@@ -393,8 +510,23 @@ int utf8_levenshteinDistance(const char *s1, const char *s2)
         int len2 = g_utf8_strlen(s2, -1);
 
         // Allocate a 2D matrix (only two rows at a time are needed)
-        int *prevRow = (int *)malloc((len2 + 1) * sizeof(int));
-        int *currRow = (int *)malloc((len2 + 1) * sizeof(int));
+        int *prevRow = malloc((len2 + 1) * sizeof(int));
+        int *currRow = malloc((len2 + 1) * sizeof(int));
+
+        if (prevRow == NULL)
+        {
+                if (currRow != NULL)
+                        free(currRow);
+                perror("malloc");
+                return 0;
+        }
+
+        if (currRow == NULL)
+        {
+                free(prevRow);
+                perror("malloc");
+                return 0;
+        }
 
         // Initialize the first row (for empty s1)
         for (int j = 0; j <= len2; j++)
@@ -417,15 +549,15 @@ int utf8_levenshteinDistance(const char *s1, const char *s2)
                         int cost = (c1 == c2) ? 0 : 1;
 
                         // Fill the current row with the minimum of deletion, insertion, or substitution
-                        currRow[j] = MIN(prevRow[j] + 1,              // deletion
-                                         MIN(currRow[j - 1] + 1,      // insertion
-                                             prevRow[j - 1] + cost)); // substitution
+                        currRow[j] = MIN(prevRow[j] + 1,              // Deletion
+                                         MIN(currRow[j - 1] + 1,      // Insertion
+                                             prevRow[j - 1] + cost)); // Substitution
                 }
 
                 // Swap rows (current becomes previous for the next iteration)
-                int *temp = prevRow;
+                int *tmp = prevRow;
                 prevRow = currRow;
-                currRow = temp;
+                currRow = tmp;
         }
 
         // The last value in prevRow contains the Levenshtein distance
@@ -444,6 +576,23 @@ int utf8_levenshteinDistance(const char *s1, const char *s2)
 #endif
 #endif
 
+char *stripFileExtension(const char *filename)
+{
+        const char *dot = strrchr(filename, '.'); // find last '.'
+        size_t length = (dot != NULL) ? (size_t)(dot - filename) : strlen(filename);
+
+        char *result = malloc(length + 1);
+        if (!result)
+        {
+                perror("malloc");
+                return NULL;
+        }
+        c_strcpy(result, filename, length);
+        result[length] = '\0';
+
+        return result;
+}
+
 // Traverses the tree and applies fuzzy search on each node
 void fuzzySearchRecursive(FileSystemEntry *node, const char *searchTerm, int threshold, void (*callback)(FileSystemEntry *, int))
 {
@@ -453,13 +602,15 @@ void fuzzySearchRecursive(FileSystemEntry *node, const char *searchTerm, int thr
         }
 
         // Convert search term, name, and fullPath to lowercase
-        char *lowerSearchTerm = g_utf8_casefold((char *)searchTerm, -1);
+        char *lowerSearchTerm = g_utf8_casefold(searchTerm, -1);
         char *lowerName = g_utf8_casefold(node->name, -1);
 
-        int nameDistance = utf8_levenshteinDistance(lowerName, lowerSearchTerm);
+        char *strippedName = stripFileExtension(lowerName);
+
+        int nameDistance = utf8_levenshteinDistance(strippedName, lowerSearchTerm);
 
         // Partial matching with lowercase strings
-        if (strstr(lowerName, lowerSearchTerm) != NULL)
+        if (strstr(strippedName, lowerSearchTerm) != NULL)
         {
                 callback(node, 0);
         }
@@ -469,42 +620,128 @@ void fuzzySearchRecursive(FileSystemEntry *node, const char *searchTerm, int thr
         }
 
         // Free the allocated memory for lowercase strings
-        free(lowerSearchTerm);
-        free(lowerName);
+        g_free(lowerSearchTerm);
+        g_free(lowerName);
+        free(strippedName);
 
         fuzzySearchRecursive(node->children, searchTerm, threshold, callback);
         fuzzySearchRecursive(node->next, searchTerm, threshold, callback);
 }
 
-FileSystemEntry *findCorrespondingEntry(FileSystemEntry *temp, const char *fullPath)
+FileSystemEntry *findCorrespondingEntry(FileSystemEntry *tmp, const char *fullPath)
 {
-        if (temp == NULL)
+        if (tmp == NULL)
                 return NULL;
-        if (strcmp(temp->fullPath, fullPath) == 0)
-                return temp;
+        if (strcmp(tmp->fullPath, fullPath) == 0)
+                return tmp;
 
-        FileSystemEntry *found = findCorrespondingEntry(temp->children, fullPath);
+        FileSystemEntry *found = findCorrespondingEntry(tmp->children, fullPath);
         if (found != NULL)
                 return found;
 
-        return findCorrespondingEntry(temp->next, fullPath);
+        return findCorrespondingEntry(tmp->next, fullPath);
 }
 
-void copyIsEnqueued(FileSystemEntry *library, FileSystemEntry *temp)
+void copyIsEnqueued(FileSystemEntry *library, FileSystemEntry *tmp)
 {
         if (library == NULL)
                 return;
 
         if (library->isEnqueued)
         {
-                FileSystemEntry *tempEntry = findCorrespondingEntry(temp, library->fullPath);
-                if (tempEntry != NULL)
+                FileSystemEntry *tmpEntry = findCorrespondingEntry(tmp, library->fullPath);
+                if (tmpEntry != NULL)
                 {
-                        tempEntry->isEnqueued = library->isEnqueued;
+                        tmpEntry->isEnqueued = library->isEnqueued;
                 }
         }
 
-        copyIsEnqueued(library->children, temp);
+        copyIsEnqueued(library->children, tmp);
 
-        copyIsEnqueued(library->next, temp);
+        copyIsEnqueued(library->next, tmp);
+}
+
+int compareFoldersByAgeFilesAlphabetically(const void *a, const void *b)
+{
+        const FileSystemEntry *entryA = *(const FileSystemEntry **)a;
+        const FileSystemEntry *entryB = *(const FileSystemEntry **)b;
+
+        // Both are directories → sort by mtime descending
+        if (entryA->isDirectory && entryB->isDirectory)
+        {
+                struct stat statA, statB;
+
+                if (stat(entryA->fullPath, &statA) != 0 || stat(entryB->fullPath, &statB) != 0)
+                        return 0;
+
+                return (int)(statB.st_mtime - statA.st_mtime); // newer first
+        }
+
+        // Both are files → sort alphabetically
+        if (!entryA->isDirectory && !entryB->isDirectory)
+        {
+                return strcasecmp(entryA->name, entryB->name);
+        }
+
+        // Put directories before files
+        return entryB->isDirectory - entryA->isDirectory;
+}
+
+void sortFileSystemEntryChildren(FileSystemEntry *parent,
+                                 int (*comparator)(const void *, const void *))
+{
+        int count = 0;
+        FileSystemEntry *curr = parent->children;
+        while (curr)
+        {
+                count++;
+                curr = curr->next;
+        }
+
+        if (count < 2)
+                return;
+
+        FileSystemEntry **entryArray = malloc(count * sizeof(FileSystemEntry *));
+
+        if (entryArray == NULL)
+        {
+                perror("malloc");
+                return;
+        }
+
+        curr = parent->children;
+        for (int i = 0; i < count; i++)
+        {
+                entryArray[i] = curr;
+                curr = curr->next;
+        }
+
+        qsort(entryArray, count, sizeof(FileSystemEntry *), comparator);
+
+        for (int i = 0; i < count - 1; i++)
+        {
+                entryArray[i]->next = entryArray[i + 1];
+        }
+        entryArray[count - 1]->next = NULL;
+        parent->children = entryArray[0];
+
+        free(entryArray);
+}
+
+void sortFileSystemTree(FileSystemEntry *root, int (*comparator)(const void *, const void *))
+{
+        if (!root)
+                return;
+
+        sortFileSystemEntryChildren(root, comparator);
+
+        FileSystemEntry *child = root->children;
+        while (child)
+        {
+                if (child->isDirectory)
+                {
+                        sortFileSystemTree(child, comparator);
+                }
+                child = child->next;
+        }
 }
