@@ -1,16 +1,8 @@
 #ifndef DIRECTORYTREE_H
 #define DIRECTORYTREE_H
 
-#include <ctype.h>
-#include <dirent.h>
-#include <regex.h>
 #include <stdbool.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
-#include "file.h"
-#include "utils.h"
+
 
 #ifndef PATH_MAX
 #define PATH_MAX 4096
@@ -46,6 +38,18 @@ FileSystemEntry *reconstructTreeFromFile(const char *filename, const char *start
 
 void fuzzySearchRecursive(FileSystemEntry *node, const char *searchTerm, int threshold, void (*callback)(FileSystemEntry *, int));
 
-void copyIsEnqueued(FileSystemEntry *library, FileSystemEntry *temp);
+void copyIsEnqueued(FileSystemEntry *library, FileSystemEntry *tmp);
+
+void sortFileSystemTree(FileSystemEntry *root, int (*comparator)(const void *, const void *));
+
+int compareFoldersByAgeFilesAlphabetically(const void *a, const void *b);
+
+int compareLibEntries(const struct dirent **a, const struct dirent **b);
+
+int compareLibEntriesReversed(const struct dirent **a, const struct dirent **b);
+
+int compareEntryNaturalReversed(const void *a, const void *b);
+
+int compareEntryNatural(const void *a, const void *b);
 
 #endif
