@@ -14,7 +14,7 @@ enum EventType
         EVENT_QUIT,
         EVENT_TOGGLEREPEAT,
         EVENT_TOGGLEVISUALIZER,
-        EVENT_TOGGLEBLOCKS,
+        EVENT_TOGGLEASCII,
         EVENT_ADDTOMAINPLAYLIST,
         EVENT_DELETEFROMMAINPLAYLIST,
         EVENT_EXPORTPLAYLIST,
@@ -38,7 +38,14 @@ enum EventType
         EVENT_PREVPAGE,
         EVENT_REMOVE,
         EVENT_SEARCH,
-        EVENT_TABNEXT
+        EVENT_NEXTVIEW,
+        EVENT_PREVVIEW,
+        EVENT_CLEARPLAYLIST,
+        EVENT_MOVESONGUP,
+        EVENT_MOVESONGDOWN,
+        EVENT_ENQUEUEANDPLAY,
+        EVENT_STOP,
+        EVENT_SORTLIBRARY
 };
 
 struct Event
