@@ -1,22 +1,19 @@
+#ifndef IMGFUNC_H
+#define IMGFUNC_H
+
 #include <chafa.h>
 #include <chafa-canvas-config.h>
-#include <stdbool.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
 #ifndef PIXELDATA_STRUCT
 #define PIXELDATA_STRUCT
-    typedef struct
-    {
+typedef struct
+{
         unsigned char r;
         unsigned char g;
         unsigned char b;
-    } PixelData;
+} PixelData;
 #endif
 
-int printInAscii(const char *pathToImgFile, int height, int width);
+int printInAscii(const char *pathToImgFile, int height);
 
 float calcAspectRatio(void);
 
@@ -25,3 +22,9 @@ unsigned char *getBitmap(const char *image_path, int *width, int *height);
 void printSquareBitmapCentered(unsigned char *pixels, int width, int height, int baseHeight);
 
 int getCoverColor(unsigned char *pixels, int width, int height, unsigned char *r, unsigned char *g, unsigned char *b);
+
+#ifdef CHAFA_VERSION_1_16
+gboolean retire_passthrough_workarounds_tmux(void);
+#endif
+
+#endif
