@@ -1,4 +1,10 @@
+#include <glib.h>
+#include "common.h"
+#include "playerops.h"
+#include "sound.h"
+#include "soundcommon.h"
 #include "mpris.h"
+
 /*
 
 mpris.c
@@ -105,8 +111,8 @@ static void handle_raise(GDBusConnection *connection, const gchar *sender,
         (void)parameters;
         (void)invocation;
         (void)user_data;
-        
-        g_dbus_method_invocation_return_value(invocation, NULL);        
+
+        g_dbus_method_invocation_return_value(invocation, NULL);
 }
 
 static void handle_quit(GDBusConnection *connection, const gchar *sender,
@@ -543,16 +549,15 @@ static gboolean get_metadata(GDBusConnection *connection, const gchar *sender,
 
                 // Build list of strings for artist
                 const gchar *artistList[2];
-                if (currentSongData->metadata->artist[0] != '\0')
+                if (g_strcmp0(currentSongData->metadata->artist, "") == 0)
                 {
-                        artistList[0] = currentSongData->metadata->artist;
-                        artistList[1] = NULL;
+                        artistList[0] = "";
                 }
                 else
                 {
-                        artistList[0] = "";
-                        artistList[1] = NULL;
+                        artistList[0] = currentSongData->metadata->artist;
                 }
+                artistList[1] = NULL;
 
                 gchar *coverArtUrl = g_strdup_printf("file://%s", currentSongData->coverArtPath);
 
@@ -676,6 +681,7 @@ static gboolean get_can_go_next(GDBusConnection *connection, const gchar *sender
         (void)user_data;
 
         CanGoNext = (currentSong == NULL || currentSong->next != NULL) ? TRUE : FALSE;
+        CanGoNext = (isRepeatListEnabled() && playlist.head != NULL) ? TRUE : CanGoNext;
 
         *value = g_variant_new_boolean(CanGoNext);
         return TRUE;
@@ -910,12 +916,12 @@ static gboolean set_property_callback(GDBusConnection *connection, const gchar *
                 }
                 else if (g_strcmp0(property_name, "LoopStatus") == 0)
                 {
-                        toggleRepeat();
+                        toggleRepeat(&(appState.uiSettings));
                         return TRUE;
                 }
                 else if (g_strcmp0(property_name, "Shuffle") == 0)
                 {
-                        toggleShuffle();
+                        toggleShuffle(&(appState.uiSettings));
                         return TRUE;
                 }
                 else if (g_strcmp0(property_name, "Position") == 0)
@@ -1115,7 +1121,7 @@ gchar *sanitizeTitle(const gchar *title)
         // Replace underscores with hyphens, otherwise some widgets have a problem
         g_strdelimit(sanitized, "_", '-');
 
-        // duplicate string otherwise widgets have a problem with certain strings for some reason
+        // Duplicate string otherwise widgets have a problem with certain strings for some reason
         gchar *sanitized_dup = g_strdup_printf("%s", sanitized);
 
         g_free(sanitized);
@@ -1255,19 +1261,23 @@ void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *a
         g_variant_builder_init(&changed_properties_builder, G_VARIANT_TYPE("a{sv}"));
         g_variant_builder_add(&changed_properties_builder, "{sv}", "Metadata", metadata_variant);
         g_variant_builder_add(&changed_properties_builder, "{sv}", "CanGoPrevious", g_variant_new_boolean((currentSong != NULL && currentSong->prev != NULL)));
-        g_variant_builder_add(&changed_properties_builder, "{sv}", "CanGoNext", g_variant_new_boolean((currentSong != NULL && currentSong->next != NULL)));
+
+        CanGoNext = (currentSong == NULL || currentSong->next != NULL) ? TRUE : FALSE;
+        CanGoNext = (isRepeatListEnabled() && playlist.head != NULL) ? TRUE : CanGoNext;
+
+        g_variant_builder_add(&changed_properties_builder, "{sv}", "CanGoNext", g_variant_new_boolean(CanGoNext));
         g_variant_builder_add(&changed_properties_builder, "{sv}", "Shuffle", g_variant_new_boolean(isShuffleEnabled()));
         g_variant_builder_add(&changed_properties_builder, "{sv}", "CanPlay", g_variant_new_boolean(length != 0 ? true : false));
         g_variant_builder_add(&changed_properties_builder, "{sv}", "CanPause", g_variant_new_boolean(length != 0 ? true : false));
-        g_variant_builder_add(&changed_properties_builder, "{sv}", "LoopStatus", g_variant_new_string(isRepeatEnabled() ? "Track" : "None"));
 
-        
+        if (isRepeatEnabled())
+                g_variant_builder_add(&changed_properties_builder, "{sv}", "LoopStatus", g_variant_new_string("Track"));
+        else if (isRepeatListEnabled())
+                g_variant_builder_add(&changed_properties_builder, "{sv}", "LoopStatus", g_variant_new_string("List"));
+        else
+                g_variant_builder_add(&changed_properties_builder, "{sv}", "LoopStatus", g_variant_new_string("None"));
 
         CanSeek = true;
-        if (currentSong != NULL && pathEndsWith(currentSong->song.filePath, "ogg"))
-        {
-                CanSeek = false;
-        }
 
         g_variant_builder_add(&changed_properties_builder, "{sv}", "CanSeek", g_variant_new_boolean(CanSeek));
 
