@@ -1,3 +1,9 @@
+#include <fcntl.h>
+#include <glib.h>
+#include <stdbool.h>
+#include <stdlib.h>
+#include <sys/time.h>
+#include <stdio.h>
 #include "notifications.h"
 
 /*
@@ -170,12 +176,12 @@ typedef struct
 
 static void bus_connection_data_ref(BusConnectionData *data)
 {
-        g_atomic_int_inc(&data->ref_count);
+        g_atomic_int_inc(&(data->ref_count));
 }
 
 static void bus_connection_data_unref(BusConnectionData *data)
 {
-        if (g_atomic_int_dec_and_test(&data->ref_count))
+        if (g_atomic_int_dec_and_test(&(data->ref_count)))
         {
                 g_main_loop_unref(data->loop);
                 g_free(data);
@@ -384,8 +390,6 @@ void cleanupDbusConnection()
                 g_object_unref(connection);
                 connection = NULL;
         }
-
-        printf("D-Bus connection successfully cleaned up.\n");
 }
 #endif
 
