@@ -1,6 +1,12 @@
 #define _XOPEN_SOURCE 700
 #define __USE_XOPEN_EXTENDED 1
 
+#include <glib.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include "file.h"
+#include "utils.h"
 #include "playlist.h"
 
 /*
@@ -18,6 +24,7 @@ Playlist related functions.
 
 const char PLAYLIST_EXTENSIONS[] = "(m3u8?)$";
 const char mainPlaylistName[] = "kew.m3u";
+const char lastUsedPlaylistName[] = "lastPlaylist.m3u";
 
 // The playlist unshuffled as it appears in playlist view
 PlayList *originalPlaylist = NULL;
@@ -25,9 +32,6 @@ PlayList *originalPlaylist = NULL;
 // The (sometimes shuffled) sequence of songs that will be played
 PlayList playlist = {NULL, NULL, 0, PTHREAD_MUTEX_INITIALIZER};
 
-// The playlist from kew.m3u
-PlayList *specialPlaylist = NULL;
-
 char search[MAX_SEARCH_SIZE];
 char playlistName[MAX_SEARCH_SIZE];
 bool shuffle = false;
@@ -67,6 +71,57 @@ void addToList(PlayList *list, Node *newNode)
         }
 }
 
+void moveUpList(PlayList *list, Node *node)
+{
+    if (node == list->head || node == NULL || node->prev == NULL)
+        return;
+
+    Node *prevNode = node->prev;
+    Node *nextNode = node->next;
+
+    if (prevNode->prev)
+        prevNode->prev->next = node;
+    else
+        list->head = node;
+
+    node->prev = prevNode->prev;
+    node->next = prevNode;
+
+    prevNode->prev = node;
+    prevNode->next = nextNode;
+
+    if (nextNode)
+        nextNode->prev = prevNode;
+    else
+        list->tail = prevNode;
+}
+
+void moveDownList(PlayList *list, Node *node)
+{
+    if (node == list->tail || node == NULL || node->next == NULL)
+        return;
+
+    Node *nextNode = node->next;
+    Node *prevNode = node->prev;
+    Node *nextNextNode = nextNode->next;
+
+    if (prevNode)
+        prevNode->next = nextNode;
+    else
+        list->head = nextNode;
+
+    nextNode->prev = prevNode;
+    nextNode->next = node;
+
+    node->prev = nextNode;
+    node->next = nextNextNode;
+
+    if (nextNextNode)
+        nextNextNode->prev = node;
+    else
+        list->tail = node;
+}
+
 Node *deleteFromList(PlayList *list, Node *node)
 {
         if (list->head == NULL || node == NULL)
@@ -146,9 +201,9 @@ void shufflePlaylist(PlayList *playlist)
         for (int j = playlist->count - 1; j >= 1; --j)
         {
                 int k = rand() % (j + 1);
-                Node *temp = nodes[j];
+                Node *tmp = nodes[j];
                 nodes[j] = nodes[k];
-                nodes[k] = temp;
+                nodes[k] = tmp;
         }
         playlist->head = nodes[0];
         playlist->tail = nodes[playlist->count - 1];
@@ -574,7 +629,7 @@ void generateM3UFilename(const char *basePath, const char *filePath, char *m3uFi
         }
 }
 
-void writeM3UFile(const char *filename, PlayList *playlist)
+void writeM3UFile(const char *filename, const PlayList *playlist)
 {
         FILE *file = fopen(filename, "w");
         if (file == NULL)
@@ -591,32 +646,52 @@ void writeM3UFile(const char *filename, PlayList *playlist)
         fclose(file);
 }
 
-void loadSpecialPlaylist(const char *directory)
+void loadPlaylist(const char *directory, const char *playlistName, PlayList *playlist)
 {
         char playlistPath[MAXPATHLEN];
 
-        if (directory[strnlen(directory, MAXPATHLEN) - 1] == '/')
+        size_t len = strnlen(directory, MAXPATHLEN);
+
+        if (len <= 0)
+                return;
+
+        if (directory[len - 1] == '/')
         {
-                snprintf(playlistPath, sizeof(playlistPath), "%s%s", directory, mainPlaylistName);
+                snprintf(playlistPath, sizeof(playlistPath), "%s%s", directory, playlistName);
         }
         else
         {
-                snprintf(playlistPath, sizeof(playlistPath), "%s/%s", directory, mainPlaylistName);
+                snprintf(playlistPath, sizeof(playlistPath), "%s/%s", directory, playlistName);
         }
 
-        specialPlaylist = malloc(sizeof(PlayList));
-        if (specialPlaylist == NULL)
+        if (playlist == NULL)
         {
                 printf("Failed to allocate memory for special playlist.\n");
                 exit(0);
         }
-        specialPlaylist->count = 0;
-        specialPlaylist->head = NULL;
-        specialPlaylist->tail = NULL;
-        readM3UFile(playlistPath, specialPlaylist);
+        playlist->count = 0;
+        playlist->head = NULL;
+        playlist->tail = NULL;
+        readM3UFile(playlistPath, playlist);
 }
 
-void saveSpecialPlaylist(const char *directory)
+void loadLastUsedPlaylist(void)
+{
+        char *configdir = getConfigPath();
+
+        loadPlaylist(configdir, lastUsedPlaylistName, &playlist);
+
+        if (originalPlaylist == NULL)
+        {
+
+                originalPlaylist = malloc(sizeof(PlayList));
+                *originalPlaylist = deepCopyPlayList(&playlist);
+        }
+        if (configdir)
+                free(configdir);
+}
+
+void saveNamedPlaylist(const char *directory, const char *playlistName, const PlayList *playlist)
 {
     if (directory == NULL)
     {
@@ -627,41 +702,54 @@ void saveSpecialPlaylist(const char *directory)
 
     int length = snprintf(playlistPath, sizeof(playlistPath), "%s", directory);
 
-    if (length < 0 || length >= (int)sizeof(playlistPath) || playlistPath[0] == '\0')
+    if (length <= 0 || length >= (int)sizeof(playlistPath) || playlistPath[0] == '\0')
     {
         return;
     }
 
     if (playlistPath[length - 1] != '/')
     {
-        snprintf(playlistPath + length, sizeof(playlistPath) - length, "/%s", mainPlaylistName);
+        snprintf(playlistPath + length, sizeof(playlistPath) - length, "/%s", playlistName);
     }
     else
     {
-        snprintf(playlistPath + length, sizeof(playlistPath) - length, "%s", mainPlaylistName);
+        snprintf(playlistPath + length, sizeof(playlistPath) - length, "%s", playlistName);
     }
 
-    if (specialPlaylist != NULL && specialPlaylist->count > 0)
+    if (playlist != NULL)
     {
-        writeM3UFile(playlistPath, specialPlaylist);
+        writeM3UFile(playlistPath, playlist);
     }
 }
 
-void savePlaylist(const char *path)
+void saveLastUsedPlaylist(void)
+{
+        char *configdir = getConfigPath();
+        saveNamedPlaylist(configdir, lastUsedPlaylistName, originalPlaylist);
+        if (configdir)
+                free(configdir);
+}
+
+void savePlaylist(const char *path, const PlayList *playlist)
 {
         if (path == NULL)
         {
                 return;
         }
 
-        if (playlist.head == NULL || playlist.head->song.filePath == NULL)
+        if (playlist->head == NULL || playlist->head->song.filePath == NULL)
                 return;
 
+        writeM3UFile(path, playlist);
+}
+
+void exportCurrentPlaylist(const char *path)
+{
         char m3uFilename[MAXPATHLEN];
 
         generateM3UFilename(path, playlist.head->song.filePath, m3uFilename, sizeof(m3uFilename));
 
-        writeM3UFile(m3uFilename, &playlist);
+        savePlaylist(m3uFilename, &playlist);
 }
 
 Node *deepCopyNode(Node *originalNode)
@@ -818,7 +906,7 @@ void createPlayListFromFileSystemEntry(FileSystemEntry *root, PlayList *list, in
 
 int isMusicFile(const char *filename)
 {
-        const char *extensions[] = {".m4a", ".aac", ".mp3", ".ogg", ".flac", ".wav", ".opus"};
+        const char *extensions[] = {".m4a", ".aac", ".mp3", ".ogg", ".flac", ".wav", ".opus", ".webm"};
 
         size_t numExtensions = sizeof(extensions) / sizeof(extensions[0]);
 
@@ -896,9 +984,9 @@ void shuffleEntries(FileSystemEntry **array, size_t n)
             size_t j = getRandomNumber(i, n - 1);
 
             // Swap entries at i and j
-            FileSystemEntry *temp = array[i];
+            FileSystemEntry *tmp = array[i];
             array[i] = array[j];
-            array[j] = temp;
+            array[j] = tmp;
         }
     }
 }
