@@ -1,4 +1,8 @@
+#include "common_ui.h"
 #include "playlist_ui.h"
+#include "songloader.h"
+#include "term.h"
+#include "utils.h"
 
 /*
 
@@ -9,6 +13,7 @@ playlist_ui.c
 */
 
 int startIter = 0;
+int previousChosenSong = 0;
 
 Node *determineStartNode(Node *head, int *foundAt, bool *startFromCurrent, int listSize)
 {
@@ -33,7 +38,7 @@ Node *determineStartNode(Node *head, int *foundAt, bool *startFromCurrent, int l
         return foundNode ? foundNode : head;
 }
 
-void preparePlaylistString(Node *node, char *buffer, int bufferSize, int shortenAmount)
+void preparePlaylistString(Node *node, char *buffer, int bufferSize)
 {
         if (node == NULL || buffer == NULL)
         {
@@ -44,18 +49,15 @@ void preparePlaylistString(Node *node, char *buffer, int bufferSize, int shorten
         char filePath[MAXPATHLEN];
         c_strcpy(filePath, node->song.filePath, sizeof(filePath));
         char *lastSlash = strrchr(filePath, '/');
-        char *lastDot = strrchr(filePath, '.');
+        size_t len = strnlen(filePath, sizeof(filePath));
 
-        if (lastSlash != NULL && lastDot != NULL && lastDot > lastSlash)
+        if (lastSlash != NULL)
         {
-                int nameLength = lastDot - lastSlash - 1;
+                int nameLength = filePath + len - (lastSlash + 1); // Length of the filename
                 nameLength = (nameLength < bufferSize - 1) ? nameLength : bufferSize - 1;
 
                 c_strcpy(buffer, lastSlash + 1, nameLength + 1);
                 buffer[nameLength] = '\0';
-                removeUnneededChars(buffer, nameLength);
-                shortenString(buffer, shortenAmount);
-                trim(buffer, MAXPATHLEN);
         }
         else
         {
@@ -68,22 +70,34 @@ int displayPlaylistItems(Node *startNode, int startIter, int maxListSize, int te
         int numPrintedRows = 0;
         Node *node = startNode;
 
-        char *buffer = (char *)malloc(MAXPATHLEN * sizeof(char));
+        int bufferSize = termWidth - indent - 12;
 
-        if (!buffer)
-        {
-                return 0;
-        }
+        PixelData rowColor;
+        rowColor.r = defaultColor;
+        rowColor.g = defaultColor;
+        rowColor.b = defaultColor;
 
         for (int i = startIter; node != NULL && i < startIter + maxListSize; i++)
         {
-                preparePlaylistString(node, buffer, MAXPATHLEN, termWidth - indent - 12);
+                if (!(ui->color.r == defaultColor && ui->color.g == defaultColor && ui->color.b == defaultColor))
+                        rowColor = getGradientColor(ui->color, i - startIter, maxListSize, maxListSize / 2, 0.7f);
+
+                char *buffer = (char *)malloc(MAXPATHLEN * sizeof(char));
+                char *filename = (char *)malloc(MAXPATHLEN * sizeof(char) + 1);
+
+                if (!buffer || !filename)
+                {
+
+                        return 0;
+                }
+                preparePlaylistString(node, buffer, MAXPATHLEN);
+
                 if (buffer[0] != '\0')
                 {
                         if (ui->useConfigColors)
                                 setTextColor(ui->artistColor);
                         else
-                                setColor(ui);
+                                setColorAndWeight(0, rowColor, ui->useConfigColors);
 
                         printBlankSpaces(indent);
 
@@ -91,30 +105,49 @@ int displayPlaylistItems(Node *startNode, int startIter, int maxListSize, int te
 
                         setDefaultTextColor();
 
+                        isSameNameAsLastTime = (previousChosenSong == chosenSong);
+
+                        if (!isSameNameAsLastTime)
+                        {
+                                resetNameScroll();
+                        }
+
+                        filename[0] = '\0';
+
                         if (i == chosenSong)
                         {
+                                previousChosenSong = chosenSong;
+
                                 *chosenNodeId = node->id;
 
+                                processNameScroll(buffer, filename, bufferSize, isSameNameAsLastTime);
+
                                 printf("\x1b[7m");
                         }
-
-                        if (currentSong != NULL && currentSong->id == node->id)
+                        else
                         {
-                                printf("\e[1m\e[39m");
+                                processName(buffer, filename, bufferSize);
                         }
 
                         if (i + 1 < 10)
                                 printf(" ");
 
-                        printf("%s \n", buffer);
+                        if (currentSong != NULL && currentSong->id == node->id)
+                        {
+                                printf("\e[4m\e[1m");
+                        }
+
+                        printf("%s\n", filename);
 
                         numPrintedRows++;
                 }
 
+                free(buffer);
+                free(filename);
+
                 node = node->next;
         }
 
-        free(buffer);
         return numPrintedRows;
 }
 
