@@ -1,11 +1,5 @@
-#include <stdbool.h>
-#include <math.h>
-#include "soundcommon.h"
+#include "appstate.h"
 #include "directorytree.h"
-#include "term.h"
-#include "common_ui.h"
-
-extern bool newUndisplayedSearch;
 
 int displaySearch(int maxListSize, int indent, int *chosenRow, int startSearchIter, UISettings *ui);
 
