@@ -1,3 +1,16 @@
+#include <fcntl.h>
+#include <poll.h>
+#include <pwd.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/ioctl.h>
+#include <sys/param.h>
+#include <sys/select.h>
+#include <termios.h>
+#include <unistd.h>
+#include "utils.h"
 #include "term.h"
 
 /*
@@ -26,7 +39,7 @@ void setTextColor(int color)
 
 void setTextColorRGB(int r, int g, int b)
 {
-        printf("\033[0;38;2;%03u;%03u;%03um", r, g, b);
+        printf("\033[0;38;2;%03u;%03u;%03um", (unsigned int)r, (unsigned int)g, (unsigned int)b);
 }
 
 void getTermSize(int *width, int *height)
@@ -219,3 +232,27 @@ int getIndentation(int terminalWidth)
         int indent = ((term_w - terminalWidth) / 2) + 1;
         return (indent > 0) ? indent : 0;
 }
+
+void enterAlternateScreenBuffer()
+{
+        // Enter alternate screen buffer
+        printf("\033[?1049h");
+}
+
+void exitAlternateScreenBuffer()
+{
+        // Exit alternate screen buffer
+        printf("\033[?1049l");
+}
+
+void enableTerminalMouseButtons()
+{
+        // Enable program to accept mouse input as codes
+        printf("\033[?1002h");
+}
+
+void disableTerminalMouseButtons()
+{
+        // Disable program to accept mouse input as codes
+        printf("\033[?1002l");
+}
